package com.fjhx.sale.flow;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.fjhx.area.utils.CustomizeAreaUtil;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.common.utils.Assert;
import com.fjhx.common.utils.ExchangeRateUtil;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.core.FlowThreadLocalUtil;
import com.fjhx.flow.enums.HandleTypeEnum;
import com.fjhx.sale.entity.contract.dto.ContractDto;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.entity.contract.po.ContractProduct;
import com.fjhx.sale.entity.contract.po.ContractProject;
import com.fjhx.sale.entity.contract.po.ContractShipment;
import com.fjhx.sale.service.contract.ContractProductService;
import com.fjhx.sale.service.contract.ContractProjectService;
import com.fjhx.sale.service.contract.ContractService;
import com.fjhx.sale.service.contract.ContractShipmentService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 外销合同流程
 *
 * @Author:caozj
 * @DATE:2023/4/3 17:38
 */
@Component
public class ContractFlow extends FlowDelegate {

    @Autowired
    private CodingRuleService codingRuleService;

    @Autowired
    private ContractService contractService;

    @Autowired
    private ContractProductService contractProductService;

    @Autowired
    private ContractProjectService contractProjectService;

    @Autowired
    private ContractShipmentService contractShipmentService;

    @Override
    public String getFlowKey() {
        return "contract_flow";
    }

    /**
     * 发起流程
     *
     * @param flowId     流程ID
     * @param submitData 采购付款数据
     * @return 业务id
     */
    @Override
    public Long start(Long flowId, JSONObject submitData) {
        ContractDto contract = submitData.toJavaObject(ContractDto.class);
        if (StringUtils.isEmpty(contract.getCurrency())) {
            throw new ServiceException("币种不能为空");
        }
        // 保存合同产品
        List<ContractProduct> contractProductList = contract.getContractProductList();

        // 赋值待处理数量
        if (CollectionUtils.isNotEmpty(contractProductList)) {
            contractProductList.forEach(item -> item.setExpendQuantity(item.getQuantity()));
        }

        return start(contract);
    }

    /**
     * 结束流程
     *
     * @param flowId     流程ID
     * @param businessId 业务ID
     * @param submitData 数据
     */
    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {

        // 通过业务ID查询合同数据
        Contract contract = contractService.getById(businessId);

        if (ObjectUtils.isEmpty(contract)) {
            throw new ServiceException("合同不存在");
        }

        // 修改采购状态为审批通过
        contract.setStatus(FlowStatusEnum.PASS.getKey());
        contract.setApprovedDate(new Date());
        contractService.updateById(contract);
    }

    public Long start(ContractDto contract) {

        long contractId = IdWorker.getId();

        // 赋值城市省份信息
        CustomizeAreaUtil.setAreaId(contract);

        contract.setId(contractId);

        //如果是变更 用用来合同号加后缀
        if (ObjectUtils.isNotEmpty(contract.getOldContractId())) {
            Contract oldContract = contractService.getById(contract.getOldContractId());
            Assert.notEmpty(oldContract, "查询不到原合同信息");
            String code = oldContract.getCode();
            Matcher matcher = Pattern.compile(".*\\((.*?)\\)$").matcher(code);
            int index = 2;
            if (matcher.find()) {
                index = (Integer.parseInt(matcher.group(1)) + 1);
                code = code.substring(0, code.lastIndexOf("("));
            }
            contract.setCode(code + "(" + index + ")");
        } else {
            contract.setCode(codingRuleService.createCode(CodingRuleEnum.CONTRACT.getKey(), contract.getBuyCorporationId()));
        }
        contract.setUserName(SecurityUtils.getUsername());
        contract.setStatus(FlowStatusEnum.UNDER_REVIEW.getKey());
        contract.setBuyCityId(contract.getCityId());
        contract.setBuyCountryId(contract.getCountryId());
        contract.setBuyProvinceId(contract.getProvinceId());
        contract.setRate(ExchangeRateUtil.getCnyToCodeRate(contract.getCurrency()));
        contractService.save(contract);

        // 保存合同产品
        List<ContractProduct> contractProductList = contract.getContractProductList();
        if (CollectionUtils.isNotEmpty(contractProductList)) {
            for (ContractProduct c : contractProductList) {
                c.setId(IdWorker.getId());
                c.setContractId(contractId);
                ObsFileUtil.saveFile(c.getFileList(), c.getId());
            }
            contractProductService.saveBatch(contractProductList);
        }

        // 保存收费项目
        List<ContractProject> contractProjectList = contract.getContractProjectList();
        if (CollectionUtils.isNotEmpty(contractProjectList)) {
            for (ContractProject c : contractProjectList) {
                c.setId(IdWorker.getId());
                c.setContractId(contractId);
            }
            contractProjectService.saveBatch(contractProjectList);
        }

        // 保存自定义出货
        List<ContractShipment> contractShipmentList = contract.getContractShipmentList();
        if (CollectionUtils.isNotEmpty(contractShipmentList)) {
            for (ContractShipment c : contractShipmentList) {
                c.setId(IdWorker.getId());
                c.setContractId(contractId);
            }
            contractShipmentService.saveBatch(contractShipmentList);
        }

        // 交接单附件列表
        ObsFileUtil.saveFile(contract.getFileList(), contractId, 1);

        // 包装指示附件列表
        ObsFileUtil.saveFile(contract.getPackageFileList(), contractId, 2);

        return contractId;
    }

    /**
     * 重新发起
     */
    public void reStart() {
        JSONObject submitData = FlowThreadLocalUtil.getCurrentData();
        ContractDto contract = submitData.toJavaObject(ContractDto.class);
        Long contractId = contract.getId();
        Assert.notEmpty(contractId, "合同id不能为空");

        if (StringUtils.isEmpty(contract.getCurrency())) {
            throw new ServiceException("币种不能为空");
        }
        // 保存合同产品
        List<ContractProduct> contractProductList = contract.getContractProductList();

        // 赋值待处理数量
        if (CollectionUtils.isNotEmpty(contractProductList)) {
            contractProductList.forEach(item -> item.setExpendQuantity(item.getQuantity()));
        }

        //-----
        // 赋值城市省份信息
        CustomizeAreaUtil.setAreaId(contract);

        contract.setId(contractId);
//        contract.setCode(codingRuleService.createCode(CodingRuleEnum.CONTRACT.getKey(), contract.getBuyCorporationId()));
        contract.setUserName(SecurityUtils.getUsername());
        contract.setStatus(FlowStatusEnum.UNDER_REVIEW.getKey());
        contract.setBuyCityId(contract.getCityId());
        contract.setBuyCountryId(contract.getCountryId());
        contract.setBuyProvinceId(contract.getProvinceId());
        contract.setRate(ExchangeRateUtil.getCnyToCodeRate(contract.getCurrency()));
        contractService.updateById(contract);

        // 保存合同产品
        if (CollectionUtils.isNotEmpty(contractProductList)) {
            for (ContractProduct c : contractProductList) {
                Assert.notEmpty(c.getId(), "合同明细id不能为空");
                c.setContractId(contractId);
                ObsFileUtil.editFile(c.getFileList(), c.getId());
            }
            contractProductService.updateBatchById(contractProductList);
        }

        // 保存收费项目
        List<ContractProject> contractProjectList = contract.getContractProjectList();
        if (CollectionUtils.isNotEmpty(contractProjectList)) {
            for (ContractProject c : contractProjectList) {
                Assert.notEmpty(c.getId(), "收费项目明细id不能为空");
                c.setContractId(contractId);
            }
            contractProjectService.updateBatchById(contractProjectList);
        }

        // 保存自定义出货
        List<ContractShipment> contractShipmentList = contract.getContractShipmentList();
        if (CollectionUtils.isNotEmpty(contractShipmentList)) {
            for (ContractShipment c : contractShipmentList) {
                Assert.notEmpty(c.getId(), "自定义出货明细id不能为空");
                c.setContractId(contractId);
            }
            contractShipmentService.updateBatchById(contractShipmentList);
        }

        // 交接单附件列表
        ObsFileUtil.editFile(contract.getFileList(), contractId, 1);

        // 包装指示附件列表
        ObsFileUtil.editFile(contract.getPackageFileList(), contractId, 2);
    }

    /**
     * 驳回方法
     */
    public void reject() {
        if (HandleTypeEnum.REJECT.equals(FlowThreadLocalUtil.getHandleTypeEnum())) {
            contractService.update(q -> q
                    .eq(Contract::getId, FlowThreadLocalUtil.getBusinessId())
                    .set(Contract::getStatus, 20)//20为驳回
                    .set(Contract::getUpdateTime, new Date())
                    .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
            );
        }
    }

}
